from setuptools import setup
from masonite.info import VERSION

setup(
    name='masonite',
    packages=[
        'masonite',
        'masonite.auth',
        'masonite.providers',
        'masonite.commands',
        'masonite.drivers',
        'masonite.managers',
        'masonite.testsuite',
        'masonite.queues',
        'masonite.contracts',
        'masonite.contracts.managers',
        'masonite.helpers',
        'masonite.middleware',
        'masonite.testing',
    ],
    version=VERSION,
    install_requires=[
        'validator.py==1.2.5',
        'cryptography==2.3',
        'bcrypt==3.1.4',
        'requests==2.19.1',
        'tldextract==2.2.0',
        'Jinja2==2.10',
        'python-dotenv==0.9.1',
        'passlib==1.7.1',
        'whitenoise==4.1',
        'pytest==3.6.0',
        'orator==0.9.7',
        'masonite-scheduler>=1.0.0,<=1.0.99',
        'masonite-events==1.0.1',
        'pendulum==1.5.1',
        'cleo>=0.6,<=0.6.99',
        'tabulate==0.8.2',
        'watchdog==0.8.3',
        'waitress==1.1.0',
        'psutil==5.4.7',
    ],
    description='The core for the Masonite framework',
    author='Joseph Mancuso',
    author_email='idmann509@gmail.com',
    url='https://github.com/MasoniteFramework/masonite',
    keywords=['masonite', 'python web framework', 'python3'],
    licence='MIT',
    classifiers=[],
    include_package_data=True,
)
